@@ -15,6 +15,7 @@ from typing import Dict, List, Tuple, Union  # , Any
 
 import matplotlib.pyplot as plt
 from matplotlib.patches import Polygon, Circle, Arc
+from matplotlib.patches import Rectangle as Rect
 
 try:
     # dessia_common >= 0.12.0
@@ -49,6 +50,7 @@ class PlotDataObject(DessiaObject):
     """ Abstract interface for DessiaObject implementation in module. """
 
     _plot_commands = "EMPTY_TEMPLATE"
+    _plot_buttons = "EMPTY_BUTTONS"
 
     def __init__(self, type_: str, name: str = '', **kwargs):
         self.type_ = type_
@@ -93,7 +95,7 @@ class Figure(PlotDataObject):
     @property
     def template(self):
         """ Get html template of current Figure object. """
-        return templates.get_html_string(command_name=self._plot_commands)
+        return templates.get_html_string(command_name=self._plot_commands, button_name=self._plot_buttons)
         # return getattr(templates, self._plot_commands)
 
     def _export_formats(self) -> List[ExportFormat]:
@@ -244,12 +246,13 @@ class PointStyle(DessiaObject):
     """
 
     def __init__(self, color_fill: str = None, color_stroke: str = None, stroke_width: float = None, size: float = None,
-                 shape: str = None, name: str = ''):
+                 shape: str = None, orientation: str = None, name: str = ''):
         self.color_fill = color_fill
         self.color_stroke = color_stroke
         self.stroke_width = stroke_width
         self.size = size  # 1, 2, 3 or 4
         self.shape = shape
+        self.orientation = orientation
         DessiaObject.__init__(self, name=name)
 
     def mpl_arguments(self):
@@ -388,7 +391,7 @@ class PointFamily(PlotDataObject):
 
     def __init__(self, point_color: str, point_index: List[int], name: str = ''):
         self.color = point_color
-        self.point_index = point_index
+        self.indices = point_index
         PlotDataObject.__init__(self, type_=None, name=name)
 
 
@@ -415,13 +418,15 @@ class Text(PlotDataObject):
     """
 
     def __init__(self, comment: str, position_x: float, position_y: float, text_style: TextStyle = None,
-                 text_scaling: bool = None, max_width: float = None, multi_lines: bool = True, name: str = ''):
+                 text_scaling: bool = None, max_width: float = None, height: float = None, multi_lines: bool = True,
+                 name: str = ''):
         self.comment = comment
         self.text_style = text_style
         self.position_x = position_x
         self.position_y = position_y
         self.text_scaling = text_scaling
         self.max_width = max_width
+        self.height = height
         self.multi_lines = multi_lines
         PlotDataObject.__init__(self, type_='text', name=name)
 
@@ -446,7 +451,9 @@ class Line2D(PlotDataObject):
     """
 
     def __init__(self, point1: List[float], point2: List[float], edge_style: EdgeStyle = None, name: str = ''):
-        self.data = point1 + point2
+        self.data = point1 + point2  # Retrocompatibility
+        self.point1 = point1
+        self.point2 = point2
         self.edge_style = edge_style
         PlotDataObject.__init__(self, type_='line2d', name=name)
 
@@ -458,13 +465,11 @@ class Line2D(PlotDataObject):
         style = self.edge_style
         if edge_style:
             style = edge_style
-        if style is None:
-            style = DEFAULT_EDGESTYLE
-
-        color = style.color_stroke.rgb
-        dashes = style.dashline
-
-        ax.axline((self.data[0], self.data[1]), (self.data[2], self.data[3]), color=color, dashes=dashes, **kwargs)
+            color = style.color_stroke.rgb
+            dashes = style.dashline
+            ax.axline((self.data[0], self.data[1]), (self.data[2], self.data[3]), color=color, dashes=dashes, **kwargs)
+        else:
+            ax.axline((self.data[0], self.data[1]), (self.data[2], self.data[3]), **kwargs)
         return ax
 
 
@@ -539,6 +544,8 @@ class Wire(PlotDataObject):
 
     def mpl_plot(self, ax=None, **kwargs):
         """ Plots using matplotlib. """
+        if not ax:
+            _, ax = plt.subplots()
         if self.edge_style:
             edge_style = self.edge_style
         else:
@@ -590,6 +597,8 @@ class Circle2D(PlotDataObject):
             edge_style = DEFAULT_EDGESTYLE
             # dashes = DEFAULT_EDGESTYLE.dashline
         args = edge_style.mpl_arguments(surface=True)
+        if 'dashes' in args:
+            args.pop("dashes")
 
         if self.surface_style:
             surface_style = self.surface_style
@@ -602,6 +611,58 @@ class Circle2D(PlotDataObject):
         return ax
 
 
+class Rectangle(PlotDataObject):
+    """ Class to draw a rectangle. """
+
+    def __init__(self, x_coord: float, y_coord: float, width: float, height: float, edge_style: EdgeStyle = None,
+                 surface_style: SurfaceStyle = None, tooltip: str = None, name: str = ''):
+        self.x_coord = x_coord
+        self.y_coord = y_coord
+        self.width = width
+        self.height = height
+        self.surface_style = surface_style
+        self.edge_style = edge_style
+        self.tooltip = tooltip
+        PlotDataObject.__init__(self, type_='rectangle', name=name)
+
+    def bounding_box(self):
+        """ Get 2D bounding box of current Circle2D. """
+        return self.x_coord, self.x_coord + self.width, self.y_coord, self.y_coord + self.height
+
+    def mpl_plot(self, ax=None, **kwargs):
+        """ Plots using matplotlib. """
+        if not ax:
+            _, ax = plt.subplots()
+        if self.edge_style:
+            edge_style = self.edge_style
+        else:
+            edge_style = DEFAULT_EDGESTYLE
+            # dashes = DEFAULT_EDGESTYLE.dashline
+        args = edge_style.mpl_arguments(surface=True)
+        if 'dashes' in args:
+            args.pop("dashes")
+
+        if self.surface_style:
+            surface_style = self.surface_style
+        else:
+            surface_style = DEFAULT_SURFACESTYLE
+
+        args.update(surface_style.mpl_arguments())
+
+        ax.add_patch(Rect([self.x_coord, self.y_coord], self.width, self.height, **args), **kwargs)
+        return ax
+
+
+class RoundRectangle(Rectangle):
+    """ Class to draw a round rectangle. """
+
+    def __init__(self, x_coord: float, y_coord: float, width: float, height: float, radius: float = 2,
+                 edge_style: EdgeStyle = None, surface_style: SurfaceStyle = None, tooltip: str = None, name: str = ''):
+        super().__init__(x_coord, y_coord, width, height, edge_style, surface_style, tooltip, name=name)
+        self.type_ = "roundrectangle"
+        self.radius = radius
+
+
 class Point2D(PlotDataObject):
     """
     A class for instantiating a point.
@@ -773,6 +834,7 @@ class Graph2D(Figure):
     """
 
     _plot_commands = "GRAPH_COMMANDS"
+    _plot_buttons = "EMPTY_BUTTONS"
 
     def __init__(self, graphs: List[Dataset], x_variable: str, y_variable: str, axis: Axis = None,
                  log_scale_x: bool = None, log_scale_y: bool = None, width: int = 750, height: int = 400,
@@ -947,6 +1009,7 @@ class Scatter(Figure):
     """
 
     _plot_commands = "SCATTER_COMMANDS"
+    _plot_buttons = "EMPTY_BUTTONS"
 
     def __init__(self, x_variable: str = None, y_variable: str = None, tooltip: Tooltip = None,
                  point_style: PointStyle = None, elements: List[Sample] = None, points_sets: List[PointFamily] = None,
@@ -985,6 +1048,7 @@ class ScatterMatrix(Figure):
     """ ScatterMatrix of a list of Samples. """
 
     _plot_commands = "MULTIPLOT_COMMANDS"
+    _plot_buttons = "MULTIPLOT_BUTTONS"
 
     def __init__(self, elements: List[Sample] = None, axes: List[str] = None, point_style: PointStyle = None,
                  surface_style: SurfaceStyle = None, width: int = 750, height: int = 400, name: str = ""):
@@ -1042,14 +1106,15 @@ class Arc2D(PlotDataObject):
     """
 
     def __init__(self, cx: float, cy: float, r: float, start_angle: float, end_angle: float, data=None,
-                 anticlockwise: bool = None, edge_style: EdgeStyle = None, name: str = ''):
+                 clockwise: bool = None, edge_style: EdgeStyle = None, name: str = ''):
         self.cx = cx
         self.cy = cy
         self.r = r
         self.start_angle = start_angle
         self.end_angle = end_angle
         self.data = data
-        self.anticlockwise = anticlockwise
+        self.anticlockwise = not clockwise
+        self.clockwise = clockwise
         self.edge_style = edge_style
         PlotDataObject.__init__(self, type_='arc', name=name)
 
@@ -1057,12 +1122,22 @@ class Arc2D(PlotDataObject):
         """ Get 2D bounding box of current Circle2D. """
         return self.cx - self.r, self.cx + self.r, self.cy - self.r, self.cy + self.r
 
+    def polygon_points(self):
+        """ Get lists of points in a merged list. """
+        points = []
+        # for primitive in self.plot_data_primitives:
+        #     points.extend(primitive.polygon_points())
+        return points
+
     def mpl_plot(self, ax=None, **kwargs):
         """ Plots using matplotlib. """
         if not ax:
             _, ax = plt.subplots()
         if self.edge_style:
-            edgecolor = self.edge_style.color_stroke
+            edgecolor = self.edge_style.mpl_arguments(surface=False)['color']
+        elif "edge_style" in kwargs:
+            edgecolor = kwargs['edge_style'].mpl_arguments(surface=False)["color"]
+            kwargs.pop("edge_style")
         else:
             edgecolor = plot_data.colors.BLACK.rgb
 
@@ -1096,6 +1171,7 @@ class Contour2D(PlotDataObject):
         self.edge_style = edge_style
         self.surface_style = surface_style
         self.tooltip = tooltip
+        self.is_filled = surface_style is not None
         PlotDataObject.__init__(self, type_='contour', name=name)
 
     def bounding_box(self):
@@ -1129,6 +1205,7 @@ class Contour2D(PlotDataObject):
         if surface_style.color_fill:
             points = self.polygon_points()
             ax.add_patch(Polygon(points, closed=True, **surface_style.mpl_arguments()), **kwargs)
+
         return ax
 
 
@@ -1148,11 +1225,12 @@ class Label(PlotDataObject):
     """
 
     def __init__(self, title: str, text_style: TextStyle = None, rectangle_surface_style: SurfaceStyle = None,
-                 rectangle_edge_style: EdgeStyle = None, name: str = ''):
+                 rectangle_edge_style: EdgeStyle = None, shape: PlotDataObject = None, name: str = ''):
         self.title = title
         self.text_style = text_style
         self.rectangle_surface_style = rectangle_surface_style
         self.rectangle_edge_style = rectangle_edge_style
+        self.shape = shape
         PlotDataObject.__init__(self, type_='label', name=name)
 
 
@@ -1180,12 +1258,14 @@ class PrimitiveGroup(Figure):
     """
 
     _plot_commands = "CONTOUR_COMMANDS"
+    _plot_buttons = "EMPTY_BUTTONS"
 
     def __init__(self, primitives: List[Union[Contour2D, Arc2D, LineSegment2D, Circle2D,
                                               Line2D, MultipleLabels, Wire, Point2D]], width: int = 750,
-                 height: int = 400, name: str = ''):
+                 height: int = 400, attribute_names: List[str] = None, name: str = ''):
         self.primitives = primitives
-        super().__init__(width=width, height=height, type_='primitivegroup', name=name)
+        self.attribute_names = attribute_names
+        super().__init__(width=width, height=height, type_='draw', name=name)
 
     def mpl_plot(self, ax=None, equal_aspect=True, **kwargs):
         """ Plots using matplotlib. """
@@ -1239,6 +1319,7 @@ class PrimitiveGroupsContainer(Figure):
     """
 
     _plot_commands = "PRIMITIVE_GROUP_CONTAINER_COMMANDS"
+    _plot_buttons = "EMPTY_BUTTONS"
 
     def __init__(self, primitive_groups: List[PrimitiveGroup], sizes: List[Tuple[float, float]] = None,
                  coords: List[Tuple[float, float]] = None, associated_elements: List[int] = None,
@@ -1277,6 +1358,7 @@ class ParallelPlot(Figure):
     """
 
     _plot_commands = "PARALLELPLOT_COMMANDS"
+    _plot_buttons = "EMPTY_BUTTONS"
 
     def __init__(self, elements: List[Sample] = None, edge_style: EdgeStyle = None, disposition: str = None,
                  axes: List[str] = None, rgbs: List[Tuple[int, int, int]] = None, width: int = 750, height: int = 400,
@@ -1323,6 +1405,7 @@ class Histogram(Figure):
     """
 
     _plot_commands = "HISTOGRAM_COMMANDS"
+    _plot_buttons = "EMPTY_BUTTONS"
 
     def __init__(self, x_variable: str, elements=None, axis: Axis = None, graduation_nb: float = None,
                  edge_style: EdgeStyle = None, surface_style: SurfaceStyle = None, width: int = 750, height: int = 400,
@@ -1346,7 +1429,7 @@ class Histogram(Figure):
         self.graduation_nb = graduation_nb
         self.edge_style = edge_style
         self.surface_style = surface_style
-        super().__init__(width=width, height=height, type_='frame', name=name)
+        super().__init__(width=width, height=height, type_='histogram', name=name)
 
 
 class MultiplePlots(Figure):
@@ -1362,6 +1445,7 @@ class MultiplePlots(Figure):
     """
 
     _plot_commands = "MULTIPLOT_COMMANDS"
+    _plot_buttons = "MULTIPLOT_BUTTONS"
 
     def __init__(self, plots: List[PlotDataObject], sizes: List[Window] = None, elements: List[Sample] = None,
                  coords: List[Tuple[float, float]] = None, point_families: List[PointFamily] = None,
@@ -1383,7 +1467,7 @@ class MultiplePlots(Figure):
         self.plots = plots
         self.sizes = sizes
         self.coords = coords
-        self.point_families = point_families
+        self.points_sets = point_families
         self.initial_view_on = initial_view_on
         super().__init__(width=width, height=height, type_='multiplot', name=name)
 
